# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: shared.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf


@dataclass(eq=False, repr=False)
class Uuid(betterproto.Message):
    structured: "UuidStructured" = betterproto.message_field(1, group="value")
    string: str = betterproto.string_field(2, group="value")


@dataclass(eq=False, repr=False)
class UuidStructured(betterproto.Message):
    most_significant_bits: int = betterproto.int64_field(1)
    least_significant_bits: int = betterproto.int64_field(2)


@dataclass(eq=False, repr=False)
class Empty(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class StreamIdentifier(betterproto.Message):
    stream_name: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class AllStreamPosition(betterproto.Message):
    commit_position: int = betterproto.uint64_field(1)
    prepare_position: int = betterproto.uint64_field(2)


@dataclass(eq=False, repr=False)
class WrongExpectedVersion(betterproto.Message):
    current_stream_revision: int = betterproto.uint64_field(
        1, group="current_stream_revision_option"
    )
    current_no_stream: "betterproto_lib_google_protobuf.Empty" = (
        betterproto.message_field(2, group="current_stream_revision_option")
    )
    expected_stream_position: int = betterproto.uint64_field(
        3, group="expected_stream_position_option"
    )
    expected_any: "betterproto_lib_google_protobuf.Empty" = betterproto.message_field(
        4, group="expected_stream_position_option"
    )
    expected_stream_exists: "betterproto_lib_google_protobuf.Empty" = (
        betterproto.message_field(5, group="expected_stream_position_option")
    )
    expected_no_stream: "betterproto_lib_google_protobuf.Empty" = (
        betterproto.message_field(6, group="expected_stream_position_option")
    )


@dataclass(eq=False, repr=False)
class AccessDenied(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class StreamDeleted(betterproto.Message):
    stream_identifier: "StreamIdentifier" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Timeout(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class Unknown(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class InvalidTransaction(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class MaximumAppendSizeExceeded(betterproto.Message):
    max_append_size: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class BadRequest(betterproto.Message):
    message: str = betterproto.string_field(1)
